// src/json/index.js
import { default as default2 } from "./get-meta-from-path.mjs";
import { default as default3 } from "./get-decorated-data-path.mjs";
export {
  default3 as getDecoratedDataPath,
  default2 as getMetaFromPath
};
//# sourceMappingURL=index.mjs.map
