var __getOwnPropNames = Object.getOwnPropertyNames;
var __commonJS = (cb, mod) => function __require() {
  return mod || (0, cb[__getOwnPropNames(cb)[0]])((mod = { exports: {} }).exports, mod), mod.exports;
};

// src/types.js
var require_types = __commonJS({
  "src/types.js"() {
  }
});
export default require_types();
//# sourceMappingURL=types.mjs.map
