import { LinkedJSONSchema, NormalizedJSONSchema } from './types/JSONSchema';
import { Options } from './';
export declare function normalize(rootSchema: LinkedJSONSchema, filename: string, options: Options): NormalizedJSONSchema;
